"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventLogger = void 0;
const date_fns_1 = require("date-fns");
const js_cookie_1 = __importDefault(require("js-cookie"));
const uuid_1 = require("uuid");
const ANONYMOUS_USER_ID_KEY = "sourcegraphAnonymousUid";
const COHORT_ID_KEY = "sourcegraphCohortId";
const FIRST_SOURCE_URL_KEY = "sourcegraphSourceUrl";
class EventLogger {
    constructor(sourcegraphURL) {
        this.sourcegraphURL = sourcegraphURL;
        this.anonymousUserID = "";
        this.cookieSettings = {
            // 365 days expiry, but renewed on activity.
            expires: 365,
            // Enforce HTTPS
            secure: true,
            // We only read the cookie with JS so we don't need to send it cross-site nor on initial page requests.
            // However, we do need it on page redirects when users sign up via OAuth, hence using the Lax policy.
            // https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Set-Cookie/SameSite
            sameSite: "Lax",
            // Specify the Domain attribute to ensure subdomains (about.sourcegraph.com) can receive this cookie.
            // https://developer.mozilla.org/en-US/docs/Web/HTTP/Cookies#define_where_cookies_are_sent
            domain: location.hostname,
        };
        this.initializeLogParameters();
    }
    /**
     * Log a `ViewStaticPage`event with the provided URL as a parameter.
     *
     * @param {string} url the URL to be logged.
     */
    log(eventName, eventProperties, publicArgument) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logToConsole(eventName, eventProperties, publicArgument);
            fetch(`${this.sourcegraphURL}/.api/graphql`, {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    Accept: "application/json",
                },
                body: JSON.stringify({
                    query: `
            mutation logEvent(
                $name: String!
                $userCookieID: String!
                $cohortID: String
                $firstSourceURL: String!
                $referrer: String
                $url: String!
                $source: EventSource!
                $argument: String
                $publicArgument: String
            ) {
                logEvent(
                    event: $name,
                    userCookieID: $userCookieID,
                    cohortID: $cohortID,
                    firstSourceURL: $firstSourceURL,
                    referrer: $referrer,
                    url: $url,
                    source: $source
                    argument: $argument
                    publicArgument: $publicArgument
                ) {
                    alwaysNil
                }
            }
        `,
                    variables: {
                        name: eventName,
                        userCookieID: this.anonymousUserID,
                        cohortID: this.cohortID || null,
                        firstSourceURL: this.getFirstSourceURL(),
                        referrer: this.getReferrer(),
                        url: window.location.href,
                        source: "STATICWEB",
                        argument: eventProperties
                            ? JSON.stringify(eventProperties)
                            : null,
                        publicArgument: publicArgument
                            ? JSON.stringify(publicArgument)
                            : null,
                    },
                }),
            }).catch((error) => {
                // Swallow errors. This should not be necessary for the Sourcegraph Cloud GraphQL API (that
                // this file logs to), which should always be up to date, but end users should not be subjected
                // to error messages.
                console.log(error);
            });
        });
    }
    logToConsole(eventName, eventProperties, publicArgument) {
        if (!localStorage || localStorage.getItem("eventLogDebug") !== "true") {
            return;
        }
        console.debug("%cEVENT %s", "color: #aaa", eventName, eventProperties, publicArgument);
    }
    /**
     * Gets the anonymous user ID and cohort ID of the user from cookies.
     * If user doesn't have an anonymous user ID yet, a new one is generated, along with
     * a cohort ID of the week the user first visited.
     *
     * If the user already has an anonymous user ID before the introduction of cohort IDs,
     * the user will not haved a cohort ID.
     *
     * If user had an anonymous user ID in localStorage, it will be migrated to cookies.
     */
    initializeLogParameters() {
        let anonymousUserID = js_cookie_1.default.get(ANONYMOUS_USER_ID_KEY);
        let cohortID = js_cookie_1.default.get(COHORT_ID_KEY);
        if (!anonymousUserID) {
            anonymousUserID = (0, uuid_1.v4)();
            cohortID = this.getPreviousMonday(new Date());
        }
        // Use cookies instead of localStorage so that the ID can be shared with subdomains (about.sourcegraph.com).
        // Always set to renew expiry and migrate from localStorage
        js_cookie_1.default.set(ANONYMOUS_USER_ID_KEY, anonymousUserID, this.cookieSettings);
        localStorage.removeItem(ANONYMOUS_USER_ID_KEY);
        if (cohortID) {
            js_cookie_1.default.set(COHORT_ID_KEY, cohortID, this.cookieSettings);
        }
        this.anonymousUserID = anonymousUserID;
        this.cohortID = cohortID;
    }
    getFirstSourceURL() {
        const firstSourceURL = this.firstSourceURL ||
            js_cookie_1.default.get(FIRST_SOURCE_URL_KEY) ||
            window.location.href;
        // Use cookies instead of localStorage so that the ID can be shared across subdomains (about.sourcegraph.com).
        // Always set to renew expiry and migrate from localStorage
        js_cookie_1.default.set(FIRST_SOURCE_URL_KEY, firstSourceURL, this.cookieSettings);
        this.firstSourceURL = firstSourceURL;
        return firstSourceURL;
    }
    getPreviousMonday(date) {
        return (0, date_fns_1.formatISO)((0, date_fns_1.startOfWeek)(date, { weekStartsOn: 1 }), {
            representation: "date",
        });
    }
    getReferrer() {
        const referrer = document.referrer;
        try {
            // 🚨 SECURITY: If the referrer is a valid Sourcegraph.com URL,
            // only send the hostname instead of the whole URL to avoid
            // leaking private repository names and files into our data.
            const url = new URL(referrer);
            if (url.hostname === "sourcegraph.com") {
                return "sourcegraph.com";
            }
            return referrer;
        }
        catch (_a) {
            return "";
        }
    }
}
exports.EventLogger = EventLogger;
//# sourceMappingURL=index.js.map